@@ -51,12 +51,17 @@ along with notmuch.  If not, see <http://www.gnu.org/licenses/>.
 
 Copyright 2010-2011 Sebastian Spaeth <Sebastian@SSpaeth.de>
 """
-from notmuch.database import Database, Query
-from notmuch.message import Messages, Message
-from notmuch.thread import Threads, Thread
-from notmuch.tag import Tags
-from notmuch.globals import (
-    nmlib,
+from .database import Database
+from .directory import Directory
+from .filenames import Filenames
+from .message import Message
+from .messages import Messages
+from .query import Query
+from .tag import Tags
+from .thread import Thread
+from .threads import Threads
+from .globals import nmlib
+from .errors import (
     STATUS,
     NotmuchError,
     OutOfMemoryError,
@@ -71,6 +76,6 @@ from notmuch.globals import (
     UnbalancedAtomicError,
     NotInitializedError,
 )
-from notmuch.version import __VERSION__
+from .version import __VERSION__
 __LICENSE__ = "GPL v3+"
 __AUTHOR__ = 'Sebastian Spaeth <Sebastian@SSpaeth.de>'
