@@ -100,7 +100,7 @@ class Query(object):
         # create reference to parent db to keep it alive
         self._db = db
         # create query, return None if too little mem available
-        query_p = Query._create(db.db_p, _str(querystr))
+        query_p = Query._create(db._db, _str(querystr))
         if not query_p:
             raise NullPointerError
         self._query = query_p
